#ifndef CINEMA_H
#define CINEMA_H 1


typedef struct Coords_t {
	int row;
	int column;
} Coords_t;


/* Functions exposed by the cinema management system to the Clients */

/* Used to initialize the server side of the application. It takes as input parameter the size of the cinema, the number of threads to be concurrently activated on the server side, and the size of the internal buffer used to enqueue user client requests. 

This function returns:
   *   0, if the initialization was successful; 
   *  -1 if the initialization failed; 
   *  -2 if the cinema has already been initialized. In this case the function has no effect.*/
int init_cinema(unsigned int num_rows, unsigned int num_cols, int num_server_threads, int buf_size);

/* returns -1 if not all seats are atomically reservable. Otherwise it returns the reservation id, encoded as an integer larger than 0.*/
int reserva(Coords_t* lugares, int dim);

/* returns -1 if not all seats are atomically reservable. Otherwise it reserves the seats for the specified number of seconds, and returns the reservation id, encoded as an integer larger than 0. */
int prereserva(Coords_t* lugares, int dim, int segundos);

/* returns -1 if the reservation does not currently exist; 0 if the pre-booking was confirmed and updated into a booking. */
int confirma_prereserva(int reservation_id);


/* returns -1 if the reservation does not currently exist; 0 if the reservation was removed and the correspondig seats made available */
int cancela(int reservation_id);

/* displays the status of the cinema. The format of the ouput must be compliant with the one generated by the file cinema/show.c */
void mostra_cinema();

#endif
